try:                    #Python 2
    from xmlrpclib import ServerProxy
    from settings import Settings
except ImportError:     #Python 3
    from xmlrpc.client import ServerProxy
    from opensubtitles.settings import Settings


class OpenSubtitles(object):
    '''OpenSubtitles API wrapper.

    Please check the official API documentation at:
    http://trac.opensubtitles.org/projects/opensubtitles/wiki/XMLRPC
    '''

    def __init__(self, language=None):
        self.xmlrpc = ServerProxy(Settings.OPENSUBTITLES_SERVER,
                                  allow_none=True)
        self.language = language or Settings.LANGUAGE
        self.token = None

    def _get_from_data_or_none(self, key):
        '''Return the key getted from data if the status is 200,
        otherwise return None.
        '''
        status = self.data.get('status').split()[0]
        return self.data.get(key) if '200' == status else None

    def login(self, username, password):
        '''Returns token is login is ok, otherwise None.
        '''
        self.data = self.xmlrpc.LogIn(username, password,
                                 self.language, Settings.USER_AGENT)
        token = self._get_from_data_or_none('token')
        if token:
            self.token = token
        return token

    def logout(self):
        '''Returns True is logout is ok, otherwise None.
        '''
        data = self.xmlrpc.LogOut(self.token)
        return '200' in data.get('status')

    def search_subtitles(self, params):
        '''Returns a list with the subtitles info.
        '''
        self.data = self.xmlrpc.SearchSubtitles(self.token, params)
        return self._get_from_data_or_none('data')

    def try_upload_subtitles(self, params):
        '''Return True if the subtitle is on database, False if not.
        '''
        self.data = self.xmlrpc.TryUploadSubtitles(self.token, params)
        return self._get_from_data_or_none('alreadyindb') == 1

    def upload_subtitles(self, params):
        '''Returns the URL of the subtitle in case that the upload is OK,
        other case returns None.
        '''
        self.data = self.xmlrpc.UploadSubtitles(self.token, params)
        return self._get_from_data_or_none('data')

    def no_operation(self):
        '''Return True if the session is actived, False othercase.

        .. note:: this method should be called 15 minutes after last request to
                  the xmlrpc server.
        '''
        data = self.xmlrpc.NoOperation(self.token)
        return '200' in data.get('status')

    def auto_update(self, program):
        '''Returns info of the program: last_version, url, comments...
        '''
        data = self.xmlrpc.AutoUpdate(program)
        return data if '200' in data.get('status') else None

    def search_movies_on_imdb(self, params):
        self.data = self.xmlrpc.SearchMoviesOnIMDB(self.token, params)
        return self.data

    def search_to_mail(self):
        # array SearchToMail( $token, array( $sublanguageid, $sublanguageid, ...), array( array( 'moviehash' => $moviehash, 'moviesize' => $moviesize), array( 'moviehash' => $moviehash, 'moviesize' => $moviesize), ...) )'
        raise NotImplementedError

    def check_subtitle_hash(self):
        # array CheckSubHash( $token, array($subhash, $subhash, ...) )
        raise NotImplementedError

    def check_movie_hash(self):
        # array CheckMovieHash( $token, array($moviehash, $moviehash, ...) )
        raise NotImplementedError

    def check_movie_hash_2(self):
        # array CheckMovieHash2( $token, array($moviehash, $moviehash, ...) )
        raise NotImplementedError

    def insert_movie_hash(self):
        # array InsertMovieHash( $token, array( array('moviehash' => $moviehash, 'moviebytesize' => $moviebytesize, 'imdbid' => $imdbid, 'movietimems' => $movietimems, 'moviefps' => $moviefps, 'moviefilename' => $moviefilename), array(...) ) )
        raise NotImplementedError

    def detect_language(self):
        # array DetectLanguage( $token, array($text, $text, ...) )
        raise NotImplementedError

    def download_subtitles(self):
        # array DownloadSubtitles( $token, array($IDSubtitleFile, $IDSubtitleFile,...) )
        raise NotImplementedError

    def report_wrong_movie_hash(self):
        # array ReportWrongMovieHash( $token, $IDSubMovieFile )
        raise NotImplementedError

    def get_subtitle_languages(self):
        # array GetSubLanguages( $language = 'en' )
        raise NotImplementedError

    def get_available_translations(self):
        # array GetAvailableTranslations( $token, $program )
        raise NotImplementedError

    def get_translation(self):
        # array GetTranslation( $token, $iso639, $format, $program )
        raise NotImplementedError

    def get_imdb_movie_details(self):
        # array GetIMDBMovieDetails( $token, $imdbid )
        raise NotImplementedError

    def insert_movie(self):
        # array InsertMovie( $token, array('moviename' => $moviename, 'movieyear' => $movieyear) )
        raise NotImplementedError

    def subtitles_vote(self):
        # array SubtitlesVote( $token, array('idsubtitle' => $idsubtitle, 'score' => $score) )
        raise NotImplementedError

    def get_comments(self):
        # array GetComments( $token, array($idsubtitle, $idsubtitle, ...))
        raise NotImplementedError

    def add_comment(self):
        # array AddComment( $token, array('idsubtitle' => $idsubtitle, 'comment' => $comment, 'badsubtitle' => $int) )
        raise NotImplementedError

    def add_request(self):
        # array AddRequest( $token, array('sublanguageid' => $sublanguageid, 'idmovieimdb' => $idmovieimdb, 'comment' => $comment ) )
        raise NotImplementedError



import os
import shutil
import tempfile
import zipfile

import requests

from opensubtitles.opensubtitles import OpenSubtitles
from opensubtitles.utils import File
from opensubz.config import Config


class OpenSubz(object):

    @staticmethod
    def search(path, language='eng'):
        print('Starting OpenSubz...\n')
        print(path)
        print('\nSearching for subtittles...')

        result = OpenSubz._download_by_hash(path, language)

        if result['arq_video'] == 0:
            print('\nI haven\'t found any video file.\n')

        if len(result['nao_achou']) > 0:
            print('\nI haven\'t found any subtitle that fits your video file\n')
            for file in result['nao_achou']:
                print(os.path.basename(file))
            print('\nDo you want to keep searching? (y/n) (Maybe the subtitles aren\'t perfectly synchronized)')
            response = str(input('> ')).lower()

            if response == 'yes' or response == 'y':
                OpenSubz._download_by_name(result['nao_achou'], language)

        input('\nPress any key to exit.')

    @staticmethod
    def _get_opensubs_token(open_subtitles):
        opensubs_token = None
        err = True
        config = Config()
        while err:
            try:
                opensubs_token = open_subtitles.login(config.email, config.password)
                if opensubs_token is not None:
                    err = False
            except Exception as e:
                print('\nAn error occurred while trying to get the opensubtitles token. Trying again...')
                print(e)
                continue

        return opensubs_token

    @staticmethod
    def _search_opensubtitles(open_subtitles, params):
        err = True
        tries = 0
        while err:
            try:
                data = open_subtitles.search_subtitles(params)
                err = False
                return data
            except:
                if tries > 5:
                    exit()
                print('\nSubtitle not found. Trying again...')
                tries += 1
                continue

    @staticmethod
    def _save_subtitles(links, diretorio):
        try:
            res = requests.get(links[0]['download'])
            res.raise_for_status()

            temp = tempfile.mkdtemp()

            nome_legenda_original = links[0]['nome-original']
            nome_legenda_zip = links[0]['nome-arquivo'][:-4] + '.zip'
            nome_legenda = links[0]['nome-arquivo'][:-4] + '.srt'
            dir_zip = os.path.join(temp, nome_legenda_zip)

            print('\nDownloading subtitle... ' + links[0]['nome-arquivo'])
            zipado = open(dir_zip, 'wb')
            for chunk in res.iter_content(8192):
                zipado.write(chunk)
            zipado.close()

            with zipfile.ZipFile(dir_zip, "r") as z:
                z.extractall(temp)

            shutil.move(os.path.join(temp, nome_legenda_original), os.path.join(diretorio, nome_legenda))
            shutil.rmtree(temp)
        except Exception as e:
            print('\nError while saving the subtitle. ' + str(e))

    @staticmethod
    def _download_by_hash(diretorio, language):
        sub = OpenSubtitles()
        token = OpenSubz._get_opensubs_token(sub)
        arq_video = 0
        nao_achou = []

        if token:
            for pastas, subpastas, nome_arquivos in os.walk(diretorio):
                for nome_arquivo in nome_arquivos:
                    if nome_arquivo.endswith('.mkv') or nome_arquivo.endswith('.mp4') or nome_arquivo.endswith('.avi'):
                        arq_video += 1

                        legenda = nome_arquivo[:-4] + '.srt'
                        if os.path.isfile(os.path.join(pastas, legenda)):
                            print('\nYou\'ve already downloaded the subtitle: ' + legenda + '. Let\'s skip this.')
                            continue

                        f = File(os.path.join(pastas, nome_arquivo))
                        fhash = f.get_hash()
                        if fhash:
                            links = []
                            size = f.size
                            data = OpenSubz._search_opensubtitles(sub, [
                                {'sublanguageid': language, 'moviefhash': fhash, 'moviebytesize': size}])

                            if len(data) == 0:
                                nao_achou.append(os.path.join(pastas, nome_arquivo))
                                print('I haven\'t found the subtitle for "%s"' % nome_arquivo)
                                continue

                            for subtitle in data:
                                links.append({'nome-original': subtitle['SubFileName'], 'nome-arquivo': nome_arquivo,
                                              'download': subtitle['ZipDownloadLink']})

                            OpenSubz._save_subtitles(links, pastas)
                        else:
                            print('Error getting the file hash: "%s"' % nome_arquivo)
            return {'arq_video': arq_video, 'nao_achou': nao_achou}
        else:
            print('Error getting opensubtitles token.')

    @staticmethod
    def _download_by_name(arquivos, language):
        if len(arquivos) > 0:
            sub = OpenSubtitles()
            token = OpenSubz._get_opensubs_token(sub)

            if token:
                for arquivo in arquivos:
                    NOME = os.path.basename(arquivo)
                    DIRETORIO = os.path.dirname(arquivo)

                    data = OpenSubz._search_opensubtitles(sub, [
                        {'sublanguageid': language, 'query': NOME}])

                    if len(data) > 0:
                        print('\nI have found these subtitles:\n')

                        for i, legenda in enumerate(data):
                            print('%d. Name: %s. Downloads: %s. Author: %s' %
                                  (i, legenda['SubFileName'], legenda['SubDownloadsCnt'], legenda['UserNickName']))

                        print('\nDigit the subtitle number you want to download.')
                        num = input('> ')

                        if num.isdigit():
                            num = int(num)
                            links = [{'nome-original': data[num]['SubFileName'], 'nome-arquivo': NOME,
                                      'download': data[num]['ZipDownloadLink']}]

                            OpenSubz._save_subtitles(links, DIRETORIO)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import sys

import opensubz.context_menu as context_menu
from opensubz.config import Config
from opensubz.core import OpenSubz


def main():
    try:

        parser = get_parser()
        args = parser.parse_args()
        config = Config()

        check_config(config)

        if args.search:
            lang = args.language if args.language else config.language
            if lang.strip():
                OpenSubz.search(args.search, lang)
            else:
                OpenSubz.search(args.search)

        elif args.config:
            set_account(config)

        elif args.install:
            context_menu.install()

        elif args.uninstall:
            context_menu.uninstall()

        else:
            parser.print_help()

    except KeyboardInterrupt:
        pass


def check_config(config):
    if (not config.email) or (not config.password):
        print("""
        We need to set your opensubtitles account first.
        If you don't have one you can register here: https://www.opensubtitles.org/newuser
        """)
        set_account(config)


def set_account(config):
    config.email = input('Email: ')
    config.password = input('Password: ')
    print('\n*** Check all available language codes: http://www.loc.gov/standards/iso639-2/php/code_list.php')
    print('*** Default: eng\n')
    config.language = input('Prefered language (ISO 639-2 code): ')
    config.save()


def get_parser():

    class MyParser(argparse.ArgumentParser):
        def error(self, message):
            sys.stderr.write('error: %s\n' % message)
            self.print_help()
            sys.exit(2)

    parser = MyParser(description="""
    opensubz 0.1.0

    Author: Daniel Sousa
    Email: sousa.dfs@gmail.com
    """, formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('-s', '--search', action='store', required=False, help='Path containing .mkv or .mp4 files to search for subtitles')
    parser.add_argument('-l', '--language', action='store', help='Filter subtitles by the language code provided.'
    '\nCheck all available language codes: http://www.loc.gov/standards/iso639-2/php/code_list.php')
    parser.add_argument('-i', '--install', action='store_true', default=False, help='Add download option in the Windows context menu')
    parser.add_argument('-u', '--uninstall', action='store_true', default=False, help='Remove download option in the Windows context menu')
    parser.add_argument('-c', '--config', action='store_true', default=False, help='Set your opensubtitles account and preferences')
    return parser

